;;#!/usr/bin/env clisp

(load "pathnames.lisp")

(defpackage :parse-data 
  (:use :cl :pathnames)
  (:export
   :load-char-list
   :load-string
   :subject-dirs
   :subject-files
   :array-contains
   :array-match
   :subjects
   :subjects-get-ids))

(in-package :parse-data)

(defun setdir (d) (setf *default-pathname-defaults* (pathname d)))
(defun dir () (pathnames:list-directory "."))

(defun trim-linefeed (s)
  (let ((orig-len (array-total-size s)))
    (if (equal (aref s (1- orig-len)) #\Return) (subseq s 0 (1- orig-len)) s)))

(defun read-line-os (stream a b)
  (let ((read-val (read-line stream a b)))
    (if (equal read-val b) b
	(trim-linefeed read-val))))

(defun list-remove-nils (l)
  (cond ((null l) nil)
	((null (car l)) (list-remove-nils (cdr l)))
	(t (cons (car l) (list-remove-nils (cdr l))))))

(defun tree-remove-nils (tree)
  (cond ((atom tree) tree)
	((equal (car tree) nil) (tree-remove-nils (cdr tree)))
	(t (cons (tree-remove-nils (car tree))
		 (tree-remove-nils (cdr tree))))))

(defun array-match (a ia b ib len)
  (cond ((<= len 0) t)
	((or (< (array-total-size a) len) (< (array-total-size b) len)) nil)
	((not (equal (aref a ia) (aref b ib))) nil)
	(t (array-match a (+ 1 ia) b (+ 1 ib) (- len 1)))))

(defun array-index-contains (a sub index)
  (cond ((> index (- (array-total-size a) (array-total-size sub))) nil)
	((array-match a index sub 0 (array-total-size sub)) index)
	(t (array-index-contains a sub (+ 1 index)))))

(defun array-contains (a sub)
    (array-index-contains a sub 0))

(defun str-contains-nocase (str substr)
  (array-contains (string-downcase str) (string-downcase substr)))

(defun path-list-remove-not-containing (d sub)
  (cond ((null d) nil)
	((array-contains (pathname-name (car d)) sub) (cons (car d) (path-list-remove-not-containing (cdr d) sub)))
	(t (path-list-remove-not-containing (cdr d) sub))))

(defun path-list-remove-containing (d sub)
  (cond ((null d) nil)
	((not (array-contains (pathname-name (car d)) sub)) (cons (car d) (path-list-remove-containing (cdr d) sub)))
	(t (path-list-remove-containing (cdr d) sub))))

(defun array-list-remove-not-containing (d sub)
  (cond ((null d) nil)
	((array-contains (car d) sub)
	 (cons (car d) (array-list-remove-not-containing (cdr d) sub)))
	(t (array-list-remove-not-containing (cdr d) sub))))

(defun array-list-remove-containing (d sub)
  (cond ((null d) nil)
	((not (array-contains (car d) sub))
	 (cons (car d) (array-list-remove-not-containing (cdr d) sub)))
	(t (array-list-remove-not-containing (cdr d) sub))))

(defun load-line-list (filename)
  (with-open-file (stream filename)
    (labels ((read-all ()
	       (let ((read-val (read-line-os stream nil nil)))
		 (if (null read-val) nil
		     (cons read-val (read-all))))))
      (read-all))))

(defun load-char-list (filename)
  (with-open-file (stream filename)
    (labels ((read-all ()
	       (let ((read-val (read-char stream nil nil)))
		 (if (null read-val) nil
		     (cons read-val (read-all))))))
      (read-all))))

(defun load-string (filename)
  (let ((string-char-list (load-char-list filename)))
    (make-array (length string-char-list)
		:element-type 'character
		:initial-contents string-char-list)))

(defun placename-to-concepts (placename)
  (cond
    ((str-contains-nocase placename "prkst")
     (list "park street"))
    ((str-contains-nocase placename "sidpac")
     (list "student" "dorm"))
    ((str-contains-nocase placename "1-115")
     (list "classroom"))
    ((str-contains-nocase placename "mass ave")
     (list "busy street"))
    ((str-contains-nocase placename "btwn 9 & 17")
     (list "university"))
    ((str-contains-nocase placename "b56")
     (list "building" "university"))
    ((str-contains-nocase placename "student center")
     (list "building" "food is near" "grocery store" "computer"))
    ((str-contains-nocase placename "ashdown")
     (list "student dorm"))
    ((str-contains-nocase placename "milwaukee")
     (list "far from home" "different city"))
    ((str-contains-nocase placename "st louis")
     (list "far from home" "different city"))
    ((str-contains-nocase placename "marvin")
     (list "famous professor"))
    ((str-contains-nocase placename "milwaukee")
     (list "far from home" "different city"))
    ((str-contains-nocase placename "hyatt")
     (list "hotel"))
    ((str-contains-nocase placename "chicago o hare")
     (list "airport" "far from home"))
    ((str-contains-nocase placename "mgh")
     (list "hospital"))
    ((str-contains-nocase placename "nh liquor barn")
     (list "liquor store" "different state"))
    ((str-contains-nocase placename "top o hub")
     (list "expensive restraunt"))
    ((str-contains-nocase placename "gov cntr")
     (list "subway" "train" "downtown" "city" "large buildings" "busy street" "government center"))
    ((str-contains-nocase placename "stata")
     (list "research" "lab" "university" "architecture"))
    ((or (str-contains-nocase placename "Strrow")
	 (str-contains-nocase placename "Storrow"))
     (list "street" "near university"))
    ((str-contains-nocase placename "abp")
     (list "french" "bagel" "sandwich" "food"))
    ((str-contains-nocase placename "chinatown")
     (list "downtown" "large buildings" "chinese food is near"))
    ((str-contains-nocase placename "apt")
     (list "apartment"))
    ((str-contains-nocase placename "mccormick")
     (list "dorm"))
    ((str-contains-nocase placename "next")
     (list "dorm"))
    ((str-contains-nocase placename "pde")
     (list "fraternity"))
    ((str-contains-nocase placename "bex")
     (list "dorm"))
    ((str-contains-nocase placename "infinite")
     (list "hallway" "corridor" "building" "university"))
    ((str-contains-nocase placename "tang")
     (list "dorm"))
    ((str-contains-nocase placename "kendal")
     (list "subway" "train"))
    ((str-contains-nocase placename "central sqr")
     (list "subway" "train"))
    ((str-contains-nocase placename "boston common")
     (list "public park"))
    ((str-contains-nocase placename "inman sqt")
     (list "subway train"))
    ((str-contains-nocase placename "beacon hill")
     (list "city street"))
    ((str-contains-nocase placename "fenway park")
     (list "baseball" "stadium"))
    ((str-contains-nocase placename "park street")
     (list "subway" "train"))
    ((str-contains-nocase placename "harvard")
     (list "university"))
    ((str-contains-nocase placename "rm495")
     (list "university" "classroom" "building"))
    ((str-contains-nocase placename "green house")
     (list "dorm"))
    ((str-contains-nocase placename "south station")
     (list "subway" "train"))
    ((str-contains-nocase placename "thirsty")
     (list "beer" "bar"))
    ((str-contains-nocase placename "porter")
     (list "subway" "train"))
    ((str-contains-nocase placename "hampshire & elm")
     (list "street"))
    ((str-contains-nocase placename "newbury")
     (list "city street" "shopping" "restraunt"))
    ((str-contains-nocase placename "mit")
     (list "university"))
    ((str-contains-nocase placename "taco bell")
     (list "fast food" "cheap"))
    ((str-contains-nocase placename "220 putnam ave")
     (list "girlfriend" "house"))
    ((or (str-contains-nocase placename "ml")
	 (str-contains-nocase placename "1cc"))
     (list "lab" "media" "university" "office"))
    (t nil)))

(defun parse-subject-log-file-line-cell (subject-log-file-line-cell cell-ids)
  (let ((area-endindex (array-index-contains subject-log-file-line-cell ", " 0)))
    (if (not (null area-endindex))
	(let ((tower-endindex (array-index-contains subject-log-file-line-cell ", " (+ 2 area-endindex))))
	  (if (not (null tower-endindex))
	      (let ((provider-endspace (array-index-contains subject-log-file-line-cell "	" (+ 2 tower-endindex))))
		(let ((provider-endindex (if (null provider-endspace) (array-total-size subject-log-file-line-cell) provider-endspace)))
		  (let ((area-str  (subseq subject-log-file-line-cell 0 area-endindex))
			(tower-str (subseq subject-log-file-line-cell (+ 2 area-endindex) tower-endindex))
			(provider  (subseq subject-log-file-line-cell (+ 2 tower-endindex) provider-endindex)))
		    (let ((area (parse-integer area-str :junk-allowed t))
			  (tower (parse-integer tower-str :junk-allowed t)))
		      (if (not (or (null area) (null tower)))
			  (list area tower provider)
			  nil)))))
	      nil))
	nil)))

(defun parse-cell-id-name-str (cell-id-str)
  (let ((cell (parse-subject-log-file-line-cell cell-id-str nil)))
    (if (not (null cell))
	(let ((i2 (array-index-contains cell-id-str "	" 0)))
	  (if (not (null i2))
	      (let ((user-tag (subseq cell-id-str (+ i2 1) (array-total-size cell-id-str))))
		(let ((extra-concepts (placename-to-concepts user-tag)))
		  (if (null extra-concepts)
		      (list cell (list user-tag))
		      (list cell (append (list user-tag) extra-concepts))))))))))

(defun parse-subject-cellid-file (subject-cellid-file)
  (mapcar 'parse-cell-id-name-str
	  (load-line-list subject-cellid-file)))

(defun parse-subject-files-cell-ids (subject-files)
  (mapcar 'parse-subject-cellid-file
	  (path-list-remove-not-containing subject-files "cellid_names_trans")))

(defun parse-log-file-line-time-stamp (log-file-line)
  (let ((year   (parse-integer (subseq log-file-line 0 4) :junk-allowed t))
	(month  (parse-integer (subseq log-file-line 4 6) :junk-allowed t))
	(day    (parse-integer (subseq log-file-line 6 8) :junk-allowed t))
	(hour   (parse-integer (subseq log-file-line 9 11) :junk-allowed t))
	(minute (parse-integer (subseq log-file-line 11 13) :junk-allowed t))
	(second (parse-integer (subseq log-file-line 13 15) :junk-allowed t)))
    (if (not (or (null year) (null month) (null day) (null hour) (null minute) (null second)))
	(list year month day hour minute second)
	nil)))

(defun parse-subject-log-file-line-event (subject-log-file-line-event cell-ids)
  (cond ((array-contains subject-log-file-line-event "area, cell, nw:")
	 (list 'cell (parse-subject-log-file-line-cell (subseq subject-log-file-line-event 16) cell-ids)))
	(t (list nil subject-log-file-line-event))))

(defun parse-subject-log-file-line (subject-log-file-line cell-ids)
  (let ((time-stamp (parse-log-file-line-time-stamp subject-log-file-line)))
    (list time-stamp (parse-subject-log-file-line-event (subseq subject-log-file-line 16) cell-ids))))

(defun parse-subject-log-file (subject-log-file cell-ids)
  (mapcar (lambda (log-file-line)
	    (parse-subject-log-file-line log-file-line cell-ids))
	  (load-line-list subject-log-file)))

(defun parse-subject-files-logs (subject-files cell-ids)
  (mapcar (lambda (subject-file)
	    (parse-subject-log-file subject-file cell-ids))
	  (path-list-remove-not-containing subject-files "log-")))

(defun parse-subject-comm-file (subject-log-file)
  (load-line-list subject-log-file))

(defun parse-subject-files-comm (subject-files)
  (mapcar 'parse-subject-comm-file
	  (path-list-remove-not-containing subject-files "comm-")))

(defun parse-subject-dir (subject-dir)
  (let ((subject-files (pathnames:list-directory (namestring subject-dir))))
    (if (null subject-files) nil
	(let ((cell-ids (parse-subject-files-cell-ids subject-files)))
	  (list cell-ids
		(parse-subject-files-logs subject-files cell-ids)
		(parse-subject-files-comm subject-files))))))

(defun subject-get-ids (subject)
  (caar subject))

(defun subject-get-placenames (subject)
  (mapcar 'cadddr (subject-get-ids subject)))

(defun caddddr (l)
  (cdddr (cdr l)))

(defun caaddddr (l)
  (cadddr (cdr l)))

(defun subject-get-mapped-concepts (subject)
  (mapcar 'caddddr (subject-get-ids subject)))

(defun subject-get-concept-map (subject)
  (mapcar (lambda (ids)
	    (list (cadddr ids) (append (list (cadddr ids)) (caaddddr ids))))
	  (subject-get-ids subject)))

(defun subject-get-logs (subject)
  (cadr subject))

(setdir "/home/neptune/Desktop/trunk/apps/lifenet/inference/data/")

(defvar current-dir)   (setf current-dir (pathnames:list-directory "."))
(defvar dirs)          (setf dirs (pathnames:list-directory "Data"))
(defvar subject-dirs)  (setf subject-dirs
			     (cadr (tree-remove-nils (mapcar (lambda (d)
							       (pathnames:list-directory
								(namestring d)))
							     dirs))))
(defvar subjects) (setf subjects
			(list-remove-nils
			 (mapcar (lambda (subject-dir)
				   (parse-subject-dir subject-dir))
				 subject-dirs)))

;(defvar all-subject-placenames) (setf all-subject-placenames
;				      (mapcar 'subject-get-placenames
;					      subjects))

;(defvar all-subject-ids) (setf all-subject-ids
;			       (mapcar 'subject-get-ids
;				       subjects))

;(defvar all-subject-concept-maps) (setf all-subject-concept-maps
;					(mapcar 'subject-get-concept-map
;						subjects))

;(print all-subject-ids)

;(print all-subject-concept-maps)

;(print (mapcar (lambda (log)
;		 (array-list-remove-not-containing log "area, cell, nw: "))
;	       (subject-get-logs (car subjects))))

(print (subject-get-logs (car subjects)))
(print (subject-get-ids (car subjects)))
