#!/usr/bin/env python

import sys
from xmlrpclib import Server

lifenet = Server("http://localhost:%d" % 8056)

root_graph = lifenet.get_root_graph() # stores most general commonsense model

# we'll use this data object to store the default state of LifeNet
default_data  = lifenet.data_new(4)

# allocate two data objects with 4 time slices each
known_data    = lifenet.data_new(4)
inferred_data = lifenet.data_new(4)

# infer the default_data state of lifenet with no specific known_data
lifenet.infer(root_graph, known_data, default_data)

result = lifenet.data_add_slices(known_data, root_graph, """
  (0 ('buy a house' 1.0))
  (1 ('proud' 1.0) ('have money' 0.0))
  (2)
  (3)
""")
if result != 0:
    print "Error adding data to known_data."
    sys.exit(-1)

print ""
print "All of known_data:"
print lifenet.data_get_contents(known_data)

lifenet.infer(root_graph, known_data, inferred_data)

# substract default probability state from inferred state to get the inferred probable difference
lifenet.data_subtract(inferred_data, default_data)

print ""
print "Top 4 assertions per slice of inferred_data:"
# display 4 nodes with largest positive difference from default_data states
print lifenet.data_get_top_n_contents(inferred_data, 4)

# cleanup server resources
#   (garbage collection should handle this if we use local data object destructor functions)
lifenet.data_delete(default_data)
lifenet.data_delete(known_data)
lifenet.data_delete(inferred_data)

