typedef class LifeNet*          LifeNetPtr;
typedef class LifeNetException* LifeNetExceptionPtr;

typedef class LifeNetLink*          LifeNetLinkPtr;
typedef class LifeNetLinkException* LifeNetLinkExceptionPtr;

typedef class LifeNetNode* LifeNetNodePtr;

#ifndef LIFENET_HPP
#define LIFENET_HPP

#include <infra.hxx>
#include <tmrf2.hpp>
#include <string.hpp>
#include <objecthash.hpp>
#include <objectarray.hpp>

#include "lifenetdata.hpp"

class LifeNetException     : public Exception {};
class LifeNetLinkException : public LifeNetException {};

const int LINK_VERSION_0_22 = 0;
const int LINK_VERSION_0_23 = 1;

class LifeNet : public TMRF2 {
  ObjectHashPtr var_hash;
public:
  LifeNetPtr     init();
  LifeNetPtr     destroy();
  StringPtr      add_node(StringPtr tag);
  StringPtr      lookup_node(StringPtr tag);
  void           add_potential(StringPtr tag0, StringPtr tag1, bool is_temporal, JPD2Ptr jpd2);
  void           add_plausibility(StringPtr tag0, StringPtr tag1, bool is_temporal,
				  real plausibility);
  bool           scan_links(int link_version, char* text);
  LifeNetPtr     load(int link_version, char* filename);
  int            data_slice_add_phrase_prob(LifeNetDataPtr tdata,
					    int            slice_index,
					    StringPtr      phrase,
					    real           prob);
  void           data_slice_add_phrase_list_state(LifeNetDataPtr tdata,
						  int            slice_index,
						  ObjectListPtr  known_phrase_list,
						  bool           state);
  LifeNetDataPtr new_phrase_list_inference_data(ObjectListPtr phrase_list,
						real          changed_threshold);
  LifeNetDataPtr new_word_inference_data(char* query_words, real  changed_threshold);
  LifeNetDataPtr new_known_data_from_phrase_list(ObjectListPtr phrase_list);
  MRF2Ptr        fwrite(FILE* fptr);
  MRF2Ptr        print();
};

#endif // LIFENET_HPP

