#!/usr/bin/env python

import SocketServer
import xmlrpcserver
import xmlrpclib
import sys
import string
import time

from lifenet import *

class LifeNetXMLServer(xmlrpcserver.RequestHandler):
    allow_reuse_address = 1
    
    dupnum = 7
    palsize = 35
    
    root_graph = -1
    
    def call(self, method, params):
        # find the appropriate method
	print "Dispatching: ", method, params
	try:
            server_method = getattr(self, method)
	except:
            raise AttributeError, "Server does not have procedure %s" % method
        return server_method(*params)
    
    def get_root_graph(self):
        """ Returns the LifeNet root graph with all basic primary commonsense knowledge."""
        return root_graph
    
    def new_graph(self):
        """ Creates a new empty LifeNet MRF2 probabilistic graph structure."""
        return lifenet_new()
    
    def add_edge(self, lifenet_id, tag0, tag1, is_temporal, plausibility):
        """ Adds edge between two LifeNet nodes with specified
        plausibility, which is converted to a JPD."""
        lifenet_add_edge(lifenet_id, tag0, tag1, is_temporal, plausibility)
        return 1

    def add_edges(self, lifenet_id, links):
        """ Adds edges between LifeNet nodes with specified
        plausibilities, which are converted to JPDs."""
        return lifenet_add_edges(lifenet_id, links);
        
    def infer(self, lifenet_id, known_data_id, inferred_data_id):
	""" Performs belief propogating inference in order to get
	probabilities for unknown concept truth."""
	lifenet_infer(lifenet_id, known_data_id, inferred_data_id)
	return 1

    def data_new(self, slice_num):
	""" Creates a new Data object for LifeNet inference."""
	return lifenet_data_new(slice_num)

    def data_delete(self, data_id):
	""" Releases all resources associated with a Data object."""
	lifenet_data_delete(data_id)
	return 1

    def data_add_slices(self, data_id, lifenet_id, slices):
        """ Adds mutliple slice data matching a graph's nodes to a data object."""
        return lifenet_data_add_slices(data_id, lifenet_id, slices)

    def data_add_datum(self, data_id, lifenet_id, slice_index, tag, probability):
        """ Add single datum with probability to a data object."""
        return lifenet_data_add_slices(data_id, lifenet_id, "(%d (\"%s\" %g))" % (slice_index, tag, probability))

    def data_add_phrases(self, lifenet_id, data_id, truth_state, phrases):
	""" Add all phrase matches within LifeNet to a Data object."""
	lifenet_data_add_phrases(lifenet_id, data_id, truth_state, phrases)
	return 1

    def data_get_contents(self, data_id):
	""" Get the probabilistic contents of a Data object."""
	return lifenet_data_get_contents(data_id)

    def data_get_top_n_contents(self, data_id, top_num_per_slice):
	""" Get the probabilistic contents of a Data object."""
	return lifenet_data_get_top_n_contents(data_id, top_num_per_slice)

    def data_subtract(self, data_id, subtract_data_id):
	""" Subtract the probability contents of one Data object from another."""
	lifenet_data_subtract(data_id, subtract_data_id)
	return 1

    def data_copy(self, data_id, src_data_id):
	""" Copy the probability contents of one Data object to another."""
	lifenet_data_copy(data_id, src_data_id)
	return 1

    def data_differentiate_by_slice(self, data_id):
	""" Differentiate the data contents by slice.  The first slice is unaffected by this operation."""
	lifenet_data_differentiate_by_slice(data_id)
	return 1


# Main method, which loads the LifeNet database
if __name__ == '__main__':
    print "Initializing Lifenet"
    root_graph = lifenet_new()
    lifenet_load(root_graph, "../lifenet-graph.lisp")
    print "Opening save files"
    logfile = open("logfile.txt", "a")
    outfile = open("output.txt", "a")
    server_up = 0
    while server_up == 0:
        try:
            server = SocketServer.TCPServer(('', 8056), LifeNetXMLServer)
            print "serving on port 8056"
            server_up = 1
        except:
            print "Port 8056 already used.  Waiting for port to clear."
            time.sleep(10)
            
#        try:
#            server = SocketServer.TCPServer(('', 8057), LifeNetXMLServer)
#            print "serving on port 8057"
#        except:
#            try:
#                server = SocketServer.TCPServer(('', 8058), LifeNetXMLServer)
#                print "serving on port 8058"
#            except:
#                print "Couldn't start LifeNet server"
#                sys.exit()
    
    server.allow_reuse_address = 1;
    print "Serving"
    try:
	server.serve_forever()
    except:
	print "Exiting due to exception"
	server.server_close()
	sys.exit(0)

