#!/usr/bin/env python

import sys
from xmlrpclib import Server

lifenet = xmlrpclib.Server("http://localhost:%d" % 8056)

root_graph = lifenet.get_root_graph() # stores most general commonsense model
my_graph   = lifenet.new_graph()      # will represent an expert LifeNet model

print "setting up expert graph"
# -> means same time slice
# => means from present slice to future slice
result = lifenet.add_edges(my_graph, """
  (=> 'have money' 'buy car'     0.5)
  (=> 'own car'    'have money' -0.5)
  (=> 'own car'    'buy car'    -0.8)
  (-> 'buy car'    'own car'     0.5)
  (-> 'own car'    'have money' -0.6)
  (-> 'own car'    'drive car'   1.0)
  (-> 'drive car'  'crash car'   0.1)
  (-> 'crash car'  'have money' -0.8)
  (-> 'drive car'  'get ticket'  0.3)
  (-> 'drive car'  'buy car'    -0.6)
  (-> 'get ticket' 'own car'    -0.8)""")
if result != 0:
    print "Error adding edges to my_graph."
    sys.exit(-1)

print "setting up expert test data"
my_default_knowledge = lifenet.data_new(4)
my_inference         = lifenet.data_new(4)
my_knowledge         = lifenet.data_new(4)

# infers the default knowledge using my_knowledge, which contains nothing right now.
lifenet.infer(my_graph, my_knowledge, my_default_knowledge)

# slice-num format      : 0, 1, 2, ...
# node-value-pair format: (node-tag truth-state)
# slice format          : (slice-num node-value-pair node-value-pair ...)
result = lifenet.data_add_slices(my_knowledge, my_graph, """
  (0 ('have money' 1) ('own car'    0) ('crash car'  0) ('drive car'  0) ('get ticket' 0))
  (1 ('have money' 0))
  (2 ('have money' 0))
  (3 ('crash car'  1) ('have money' 0))""")
if result != 0:
    print "Error adding data to my_knowledge."
    sys.exit(-1)

# infers the default knowledge using my_knowledge, which contains nothing right now.
lifenet.infer(my_graph, my_knowledge, my_inference)

print lifenet.data_get_top_n_contents(my_inference, 3)

# delete data (not necessary if server is shut down periodically)
lifenet.data_delete(my_default_knowledge)
lifenet.data_delete(my_inference)
lifenet.data_delete(my_knowledge)

