typedef class LifeNet*          LifeNetPtr;
typedef class LifeNetException* LifeNetExceptionPtr;

typedef class LifeNetLink*          LifeNetLinkPtr;
typedef class LifeNetLinkException* LifeNetLinkExceptionPtr;

typedef class LifeNetNode* LifeNetNodePtr;

#ifndef LIFENET_HPP
#define LIFENET_HPP

#include <infra.hxx>
#include <tmrf.hpp>
#include <string.hpp>
#include <objecthash.hpp>
#include <objectarray.hpp>

#include "lifenetdata.hpp"
#include "logic.hpp"

class LifeNetException     : public Exception {};
class LifeNetLinkException : public LifeNetException {};

const int LINK_VERSION_0_22 = 0;
const int LINK_VERSION_0_23 = 1;

class LifeNet : public TMRF {
  ObjectHashPtr var_hash;
public:
  virtual ObjectPtr destroy()          { return _destroy(); };
  //virtual ObjectPtr new_copy()         { return _new_copy(); };
  virtual ObjectPtr fwrite(FILE* fptr) { return _fwrite(fptr); };
  virtual ObjectPtr print()            { return _print(); };
public:
  LifeNetPtr      init();
  ObjectPtr      _destroy();
  StringPtr       add_tag(StringPtr tag);
  StringPtr       lookup_tag(StringPtr tag);
  void            add_correlation_factor(pointer n0, pointer n1, real correlation);
  void            add_correlation_pair(StringPtr tag0, StringPtr tag1, bool is_temporal, real correlation);
  bool            scan_links(int link_version, char* text);
  LifeNetPtr      load(int link_version, char* filename);
  int             data_slice_add_phrase_prob(LifeNetDataPtr tdata,
					     int            slice_index,
					     StringPtr      phrase,
					     real           prob);
  void            data_slice_add_phrase_list_state(LifeNetDataPtr tdata,
						   int            slice_index,
						   ObjectListPtr  known_phrase_list,
						   bool           state);
  LifeNetDataPtr  new_phrase_list_inference_data(ObjectListPtr phrase_list,
						 real          changed_threshold);
  LifeNetDataPtr  new_word_inference_data(char* query_words, real  changed_threshold);
  LifeNetDataPtr  new_known_data_from_phrase_list(ObjectListPtr phrase_list);
  ObjectPtr      _fwrite(FILE* fptr);
  ObjectPtr      _print();
};

#endif // LIFENET_HPP

