#include "lifenetdata.hpp"
#include "parser.hpp"
#include <stdlib.h>
#include <string.hpp>
#include "lifenet.hpp"

void slice_parse_error() {
  throw (LifeNetDataExceptionPtr)(new LifeNetDataException)->
    init(__FILE__, __LINE__, "slice_parse_error", "error parsing slice");
}

int LifeNetData::scan_slice(LifeNetPtr lifenet, char* text) {
  int i = 0;
  ObjectListPtr known_phrase_list = (new ObjectList)->init();
  StringPtr tag = (new String)->init("");
  try {
    i = find_nonwhitespace(text);
    int j;
    if(text[i] != '(') {
      slice_parse_error();
    }
    i++;
    int slice_index;
    j = sure_parse_int(text+i, &slice_index);
    
    i += j;
    //printf("  slice_index = %d\n", slice_index);
    i += find_nonwhitespace(text+i);
    while(text[i] != ')') {
      if(text[i] != '(') {
	slice_parse_error();
      }
      i++;
      i += find_nonwhitespace(text+i);
      if(text[i] != '\'' && text[i] != '"') {
	slice_parse_error();
      }
      i += sure_parse_string(text+i, tag);
      //printf("tag: \"%s\"\n", tag->get_str());
      i += find_nonwhitespace(text+i);
      real prob;
      i += sure_parse_real(text+i, &prob);
      //printf("prob: %g\n", prob);
      
      int total_nodes_affected =
	lifenet->data_slice_add_phrase_prob(this, slice_index, tag, prob);
      
      printf("total_nodes_affected = %d\n", total_nodes_affected);
      
      if(text[i] != ')') {
	slice_parse_error();
      }
      i++;
      i += find_nonwhitespace(text+i);
    }
    i++;
    i += find_nonwhitespace(text+i);
    //printf("scanned data slice\n");
  } catch(ParserExceptionPtr e) {
    i = 0;
    delete e->destroy();
    slice_parse_error();
  }
  delete tag->destroy();
  delete known_phrase_list->destroy();
  return i;
}

bool LifeNetData::scan_slices(LifeNetPtr lifenet, char* text) {
  int i = 0;
  bool success = true;
  while(success && text[i] != 0) {
    try {
      i += scan_slice(lifenet, text+i);
    } catch(LifeNetDataExceptionPtr e) {
      success = false;
      delete e->destroy();
    }
  }
  return success;
}
