#ifndef PARSER_HPP
#define PARSER_HPP

#include <exception.hxx>
#include <string.hpp>
#include <stdlib.h>

typedef class ParserException : public Exception {}* ParserExceptionPtr;

#define PARSER_EXCEPT(func, desc) throw (ParserExceptionPtr)(new ParserException)->\
  init(__FILE__, __LINE__, func, desc);


inline bool char_is_whitespace(char ch) {
  return ch == ' ' || ch == '\n' || ch == '\r' || ch == '\t';
}

inline int char_is_digit(char ch) {
  return ch >= '0' && ch <= '9';
}

inline int char_is_sign(char ch) {
  return ch == '-' || ch == '+';
}

inline int char_is_integer(char ch) {
  return char_is_digit(ch) || char_is_sign(ch);
}

inline int find_nonwhitespace(char* str) {
  int i = 0;
  while(char_is_whitespace(str[i])) { i++; }
  return i;
}

inline int find_char_or_null(char* str, char ch) {
  int i = 0;
  while(str[i] != ch && str[i] != 0) { i++; }
  return i;
}

inline int parse_int(char* str, int* value) {
  char* end_ptr;
  *value = strtol(str, &end_ptr, 10);
  return ((int)end_ptr - (int)str);
}

inline int sure_parse_int(char* str, int* value) {
  int i = parse_int(str, value);
  if(i == 0) {
    PARSER_EXCEPT("sure_parse_int", "couldn't parse integer");
  }
  return i;
}

inline int parse_real(char* str, real* value) {
  char* end_ptr;
  *value = strtod(str, &end_ptr);
  return ((int)end_ptr - (int)str);
}

inline int sure_parse_real(char* str, real* value) {
  int i = parse_real(str, value);
  if(i == 0) {
    PARSER_EXCEPT("sure_parse_real", "couldn't parse real number");
  }
  return i;
}

inline int parse_string(char* str, StringPtr value) {
  int i = find_nonwhitespace(str);
  char quote = str[i++];
  if(quote != '\'' && quote != '"') {
    return 0;
  }
  int j = i + find_char_or_null(str+i, quote);
  if(str[j] == 0) {
    return 0;
  }
  value->set_str(str+i, j-i);
  i = j + 1;
  return i;
}

inline int sure_parse_string(char* str, StringPtr value) {
  int i = parse_string(str, value);
  if(i == 0) {
    PARSER_EXCEPT("sure_parse_string", "couldn't parse string");
  }
  return i;
}

#endif // PARSER_HPP
