(defpackage :Visualize
  (:use :cl :cpp :classutil :miscutils :cppexecutable :makefile :opengl :infra :stdfont :syswindow :xwindows)
  (:export))
(in-package :Visualize)

(defune gl_cursor ()
  "GLCursor")

(defune gl_cursor-init (font)
  (cpp_object-init (gl_cursor) font))

(defune gl_cursor-alloc (font)
  (cpp_object-alloc (gl_cursor) font))

(defune draw-rect (x0 y0 x1 y1)
  (cpp_progn
   (gl_begin-quads)
   (gl_vertex_3f x0 y0 0)
   (gl_vertex_3f x1 y0 0)
   (gl_vertex_3f x1 y1 0)
   (gl_vertex_3f x0 y1 0)
   (gl_end)))

(defune draw-box (x y width height)
  (cpp_progn
   (gl_begin-line_strip)
   (gl_vertex_3f x y 0)
   (gl_vertex_3f (cpp_+ x width) y 0)
   (gl_vertex_3f (cpp_+ x width) (cpp_+ y height) 0)
   (gl_vertex_3f x (cpp_+ y height) 0)
   (gl_vertex_3f x y 0)
   (gl_end)))

(defune color-new (red green blue alpha)
  `(:color ,red ,green ,blue ,alpha))

(defune colorp (this)
  (and (listp this) (eq (first this) :color)))

(defune color-red (this)
  (second this))

(defune color-green (this)
  (third this))

(defune color-blue (this)
  (fourth this))

(defune color-alpha (this)
  (fifth this))

(defune gl_color (this)
  (gl_color_4f (color-red   this)
	       (color-green this)
	       (color-blue  this)
	       (color-alpha this)))  

(defune gl_color_flicker (color &key (amount 0.1))
  (cpp_block
   (cpp_defvar (cpp_real) "intensity" (cpp_random-real (cpp_- 1 amount) 1))
   (gl_color_4f (cpp_* "intensity" (color-red   color))
		(cpp_* "intensity" (color-green color))
		(cpp_* "intensity" (color-blue  color))
		(color-alpha color))))

(defune draw-border (x y width height border_width &key
		       (outer_color (color-new 1 1 1 1))
		       (inner_color (color-new 0 0 0 0)))
  (cpp_progn
   (gl_begin-quads)
   ;; top
   (gl_color outer_color)
   (gl_vertex_3f x y 0)
   (gl_vertex_3f (cpp_+ x width) y 0)
   (gl_color inner_color)
   (gl_vertex_3f (cpp_- (cpp_+ x width) border_width) (cpp_+ y border_width) 0)
   (gl_vertex_3f (cpp_+ x border_width) (cpp_+ y border_width) 0)
   ;; left
   (gl_color outer_color)
   (gl_vertex_3f x (cpp_+ y height) 0)
   (gl_vertex_3f x y 0)
   (gl_color inner_color)
   (gl_vertex_3f (cpp_+ x border_width) (cpp_+ y border_width) 0)
   (gl_vertex_3f (cpp_+ x border_width) (cpp_- (cpp_+ y height) border_width) 0)
   ;; right
   (gl_color outer_color)
   (gl_vertex_3f (cpp_+ x width) y 0)
   (gl_vertex_3f (cpp_+ x width) (cpp_+ y height) 0)
   (gl_color inner_color)
   (gl_vertex_3f (cpp_- (cpp_+ x width) border_width) (cpp_- (cpp_+ y height) border_width) 0)
   (gl_vertex_3f (cpp_- (cpp_+ x width) border_width) (cpp_+ y border_width) 0)
   ;; bottom
   (gl_color outer_color)
   (gl_vertex_3f (cpp_+ x width) (cpp_+ y height) 0)
   (gl_vertex_3f x (cpp_+ y height) 0)
   (gl_color inner_color)
   (gl_vertex_3f (cpp_+ x border_width) (cpp_- (cpp_+ y height) border_width) 0)
   (gl_vertex_3f (cpp_- (cpp_+ x width) border_width) (cpp_- (cpp_+ y height) border_width) 0)
   (gl_end)))

(defune draw-double_border (x y width height border_width &key
			      (outer_color (color-new 1 1 1 1))
			      (inner_color (color-new 0 0 0 0)))
  (cpp_progn
   (draw-border x y width height (cpp_/ border_width 2.0) :outer_color outer_color :inner_color inner_color)
   (draw-border (cpp_+ x (cpp_/ border_width 2.0))
		(cpp_+ y (cpp_/ border_width 2.0))
		(cpp_- width border_width)
		(cpp_- height border_width)
		(cpp_/ border_width 2.0) :outer_color inner_color :inner_color outer_color)))

(defune draw-printf (x y format_str &rest args)
  (cpp_progn
   (gl_cursor-set_pos "stdcursor" x y 0)
   (eval (append `(gl_cursor-printf "stdcursor" ,format_str) args))))

(defune def_usecond_var (usecond_var)
  (cpp_progn
   (cpp_defvar (cpp_real) usecond_var)
   (cpp_block
    (cpp_defvar (cpp_timeval) "usecond_timeval")
    (cpp_gettimeofday (cpp_address_of "usecond_timeval") (cpp_nil))
    (cpp_setf usecond_var (cpp_+ (cpp_cast_to_type (cpp_real)
						   (cpp_timeval-tv_sec "usecond_timeval"))
				 (cpp_cast_to_type (cpp_real)
						   (cpp_/ (cpp_cast_to_type (cpp_real)
									    (cpp_timeval-tv_usec "usecond_timeval")) 1000000.0)))))))

(let ((xres 1280)
      (yres 1024))
  
  (defune draw-lifenet (x y width height)
    (cpp_block
     (gl_disable (gl_texture_2d))
     (gl_disable (gl_texture))
     (draw-double_border x y width height 8
			 :outer_color (color-new 0 0    0   0)
			 :inner_color (color-new 0.5 0.5 0.0 1))
     (gl_enable (gl_texture_2d))
     (gl_enable (gl_texture))
     (gl_color (color-new 1 1 1 1))
     (draw-printf (cpp_+ x 8) (cpp_+ y 8 (cpp_/ yres 64)) (cpp_constant_string "LifeNet"))
     (cpp_dotimes "index" (cpp_/ (cpp_- height 24 (cpp_/ yres 64)) (cpp_/ yres 64))
		  (cpp_progn
		   (gl_color (color-new 1 1 1 1))
		   (draw-printf (cpp_+ x 8)
				(cpp_+ y 24 (cpp_/ yres 64) (cpp_* "index" (cpp_/ yres 64)))
				(cpp_constant_string "knowledge phrase"))))))
  
  (defune draw-person (x y width height)
    (cpp_block
     (gl_disable (gl_texture_2d))
     (gl_disable (gl_texture))
     (draw-double_border x y width height 16
			 :outer_color (color-new 0 0    0   0)
			 :inner_color (color-new 0 0.25 0.5 1))
     (gl_enable (gl_texture_2d))
     (gl_enable (gl_texture))
     (gl_color (color-new 1 1 1 1))
     (draw-printf  (cpp_+ x 16) (cpp_+ y 32) (cpp_constant_string "Person"))
     (draw-lifenet (cpp_+ x 16) (cpp_+ y 32 (cpp_* 0.0 (cpp_/ (cpp_- height 48) 3.0))) (cpp_- width 32) (cpp_/ (cpp_- height 48) 3.0))
     (draw-lifenet (cpp_+ x 16) (cpp_+ y 32 (cpp_* 1.0 (cpp_/ (cpp_- height 48) 3.0))) (cpp_- width 32) (cpp_/ (cpp_- height 48) 3.0))
     (draw-lifenet (cpp_+ x 16) (cpp_+ y 32 (cpp_* 2.0 (cpp_/ (cpp_- height 48) 3.0))) (cpp_- width 32) (cpp_/ (cpp_- height 48) 3.0))))
     ;;(cpp_dotimes "index" (cpp_/ (cpp_- height 64) (cpp_/ yres 64))
     ;;		  (cpp_progn
     ;;		   (gl_color (color-new 1 1 1 1))
     ;;		   (draw-printf (cpp_+ x 16)
     ;;				(cpp_+ y 64 (cpp_* "index" (cpp_/ yres 64)))
     ;;				(cpp_constant_string "knowledge phrase"))))))
  
  (defmacroe cpp_simulation (num &key globals locals update)
    `(let ((element_num ,num))
       ,(append `(cpp_progn
		  (cpp_static (cpp_defvar (cpp_bool) "first_time" (cpp_bool-true))))
		(mapcar (lambda (assign)
			  `(cpp_static (cpp_defvar ,(second assign)
						   ,(cpp_symbol (first assign))
						   ,(third assign))))
			globals)
		(mapcar (lambda (assign)
			  `(cpp_static (cpp_defvar (cpp_pointer ,(second assign))
						   ,(cpp_symbol (first assign))
						   (cpp_array-new ,(second assign) ,num))))
			locals)
		`((cpp_if "first_time"
			  (cpp_progn
			   (cpp_setf "first_time" (cpp_bool-false))
			   (cpp_dotimes "j" element_num
					,(append `(cpp_progn)
						 (mapcar (lambda (assign)
							   `(cpp_setf (cpp_aref ,(cpp_symbol (first assign)) "j") ,(third assign)))
							 locals)))
			   )))
		(let ((bound_update (print update)))
		  (mapc (lambda (assign)
			  (setf bound_update (exp-simple_bind bound_update (first assign) (cpp_symbol (first assign)))))
			globals)
		  (mapc (lambda (assign)
			  (setf bound_update (exp-simple_bind bound_update (first assign) (cpp_aref (cpp_symbol (first assign)) "i"))))
			locals)
		  `((cpp_dotimes "i" ,num
				 ,(print bound_update)))
		  ))))
  
  (defmacroe visualize_simulation (num &key globals locals update)
    `(cpp_simulation ,num
		     :globals ,(append `((ax        (cpp_real) 0.01)
					 (ay        (cpp_real) 0.01)
					 (drag_x    (cpp_real) 0.2)
					 (drag_y    (cpp_real) 0.2)
					 (dist      (cpp_real) 0)
					 (dist_x    (cpp_real) 0)
					 (dist_y    (cpp_real) 0))
				       globals)
		     :locals ,(append `((tx (cpp_real) (cpp_random-real (- 0 (/ xres 2)) (/ xres 2)))
					(ty (cpp_real) (cpp_random-real (- 0 (/ yres 2)) (/ yres 2)))
					(vx (cpp_real) 0)
					(vy (cpp_real) 0)
					(gx (cpp_real) (cpp_random-real (- 0 (/ xres 2)) (/ xres 2)))
					(gy (cpp_real) (cpp_random-real (- 0 (/ yres 2)) (/ yres 2))))
				      locals)
		     :update (cpp_progn
			      (cpp_setf dist_x (cpp_- tx gx))
			      (cpp_setf dist_y (cpp_- ty gy))
			      (cpp_setf dist   (cpp_sqrt (cpp_+ (cpp_* dist_x dist_x) (cpp_* dist_y dist_y))))
			      (cpp_cond ((cpp_< tx gx)
					 (cpp_setf vx (cpp_+ vx (cpp_* dist ax))))
					((cpp_> tx gx)
					 (cpp_setf vx (cpp_- vx (cpp_* dist ax)))))
			      (cpp_cond ((cpp_< ty gy)
					 (cpp_setf vy (cpp_+ vy (cpp_* dist ay))))
					((cpp_> ty gy)
					 (cpp_setf vy (cpp_- vy (cpp_* dist ay)))))
			      (cpp_setf vx (cpp_* (cpp_- 1 drag_x) vx))
			      (cpp_setf vy (cpp_* (cpp_- 1 drag_y) vy))
			      (cpp_setf tx (cpp_+ tx vx))
			      (cpp_setf ty (cpp_+ ty vy))
			      ,update)))
  
  (defune visualize_homing_text ()
    (visualize_simulation 24
			  :globals
			  ((str_width (cpp_real) 0))
			  :locals
			  nil
			  :update
			  (cpp_progn
			   (cpp_setf str_width (gl_cursor-get_str_width "stdcursor" (cpp_constant_string "Test")))
			   (gl_disable (gl_texture_2d))
			   (gl_color_4f 0 0.25 0.5 1)				
			   (draw-box (cpp_- gx (cpp_/ str_width 2) 1) (cpp_- gy 5 1)
				     (cpp_+ str_width 2) (cpp_+ 10 2))
			   (gl_enable (gl_texture_2d))
			   (gl_color_4f 1 1 1 1)
			   (gl_cursor-set_pos "stdcursor" (cpp_- tx (cpp_/ str_width 2)) (cpp_- ty 5) 0)
			   (gl_cursor-draw_str "stdcursor" (cpp_constant_string "Test")))))
  
  (defune visualize_tick ()
    (cpp_progn
     ;;(gl_clear_stencil 0)
     (gl_clear_color   0 0 0 0)
     ;;(gl_clear         (cpp_bit_or (gl_color_buffer_bit) (gl_depth_buffer_bit) (gl_stencil_buffer_bit)))
     (gl_clear         (gl_color_buffer_bit))
     (gl_matrix_mode (gl_projection))
     (gl_load_identity)
     (gl_matrix_mode (gl_modelview))
     (gl_load_identity)
     ;;(gl_scalef 0.1 0.1 0.1)
     ;;(gl_scalef 0.004 0.003 0.003)
     
     (gl_color_4f 1 1 1 1)
     
     ;;(gl_cursor-bind         "stdcursor")
     ;;(draw-rect -1 -1 1 1)
     
     ;;(gl_disable    "GL_CULL_FACE")
     ;;(gl_disable    (gl_blend))
     
     ;;(gl_disable   (gl_stencil_test))
     ;;(gl_Disable   (GL_LIGHTING);
     ;;(gl_Shade_Model (GL_FLAT))
     
     (cpp_static (cpp_defvar (cpp_int) "frame_count" 0))
     (cpp_incr "frame_count")
     
     (gl_enable (gl_texture))
     (gl_enable (gl_texture_2d))
     (gl_shade_model (gl_smooth))
     (gl_translate_f -1 1 0)
     (gl_scale_f (/ 2.0 xres) (- (/ 2.0 yres)) 2.0)
     (gl_cursor-set_width        "stdcursor" (/ yres 64))
     (gl_cursor-set_height       "stdcursor" (/ yres 64))
     (gl_cursor-set_up_vector    "stdcursor" 0 -1 0)
     (gl_cursor-set_right_vector "stdcursor" 1 0 0)
     
     
     (draw-person (* 0.0 (- xres (/ xres 3.0))) 0 (/ xres 3.0) yres)
     (draw-person (* 0.5 (- xres (/ xres 3.0))) 0 (/ xres 3.0) yres)
     (draw-person (* 1.0 (- xres (/ xres 3.0))) 0 (/ xres 3.0) yres)
     
     (cpp_static (cpp_defvar (cpp_real) "last_seconds" 0))
     (def_usecond_var "current_seconds")
     (cpp_defvar (cpp_real) "elapsed_seconds" (cpp_- "current_seconds" "last_seconds"))
     (cpp_if (cpp_!= "elapsed_seconds" 0)
	     (draw-printf 0 (- yres 1) (cpp_constant_string "%g F.P.S.") (cpp_/ 1.0 "elapsed_seconds"))
	     (draw-printf 0 (- yres 1) (cpp_constant_string "NaN")))
     (cpp_setf "last_seconds" "current_seconds")

     ;;glDisable(GL_STENCIL_TEST);
     
     )))

(defune gl_texture-bind (this)
  (cpp_semicolon (cpp_ptr_funcall this "bind")))

(defune gl_font-get_texture (this)
  (cpp_ptr_funcall this "get_texture"))

(defune gl_font-bind (this)
  (gl_texture-bind (gl_font-get_texture this)))

(defune gl_font-init_file (this filename quality)
  (cpp_semicolon (cpp_ptr_funcall this "init" filename quality)))

(defune gl_font-init_data (this width height rgba_data invert_color quality)
  (cpp_semicolon (cpp_ptr_funcall this "init" width height rgba_data invert_color quality)))

(defune gl_cursor-init (this font)
  (cpp_semicolon (cpp_ptr_funcall this "init" font)))

(defune gl_cursor-get_font (this)
  (cpp_ptr_funcall this "get_font"))

(defune gl_cursor-bind (this)
  (gl_font-bind (gl_cursor-get_font this)))

(defune visualize-setup_cursor ()
  (cpp_progn
   (cpp_if (cpp_null "stdcursor")
	   (cpp_setf "stdcursor" (gl_cursor-alloc (stdfont-new_font 1 1))))
   (gl_cursor-use_font_num "stdcursor" 0)
   (gl_cursor-set_pos      "stdcursor" 0 0 0)
   (gl_cursor-bind         "stdcursor")))

(defvare *use_syswindow* nil)

(defune visualize_cpp_code-xwindows ()
  (cpp_progn ;; the cpp code
   (package-cpp_includes :visualize)
   (cpp_include "xwindows.hpp")
   (cpp_include "sys/time.h")
   (cpp_include "time.h")
   (cpp_defobjptr (gl_cursor) "stdcursor" (cpp_nil))
   ;;(cpp_defvar "GLuint" "stdfont_texture")
   ;;   (cpp_function "main" :type (cpp_int) :code
   ;;		 (cpp_progn
   ;;		  (display_window "visualize.lisp"
   ;;				  :on_tick
   ;;				  (visualize_tick))
   (cpp_defune main :type (cpp_int) :args (((cpp_int) argc) ((cpp_char**) argv))
	       :code
	       (cpp_progn
		(cpp_if (cpp_! (xwindows_init))
			(cpp_return -1))
		;;(visualize-setup_texture)
		(visualize-setup_cursor)
		(cpp_while (xwindows_handle_events)
			   (cpp_progn
			    ;;(gl_bind_texture (gl_texture_2d) "stdfont_texture")
			    (visualize_tick)
			    ;;"xwindows_draw_gl_scene();"
			    (xwindows_gl_swap_buffers)))
		(xwindows_kill_gl_window)
		(cpp_return 0)))))

(defune visualize_cpp_code-syswindow ()
  (cpp_progn ;; the cpp code
   (package-cpp_includes :visualize)
   (cpp_defobjptr (gl_cursor) "stdcursor" (cpp_nil))
   (cpp_defune main :type (cpp_int) :args (((cpp_int) argc) ((cpp_char**) argv))
	       :code
	       (display_window "visualize.lisp"
			       :on_tick
			       (visualize_tick)))))

(defune visualize_cpp_code ()
  (if *use_syswindow*
      (visualize_cpp_code-syswindow)
    (visualize_cpp_code-xwindows)))

(defproject :visualize
  :packages (append `(:stdfont :infra)
		    (if *use_syswindow*
			`(:syswindow)
		      `(:xwindows)))
  :libs     (append `()
		    (if *use_syswindow*
			`()
		      `("stdfont" "xwindows")))
  :base_dir (format nil "~A/apps/lifenet/visualize" cl-user:*think-directory-root*)
  :cpp_code (visualize_cpp_code))

